/* Copyright (C) 2005 Andrei Krger */


void ocd_update_template(char *fn, char *template, rect page, double resolution);

typedef struct {
  long x, y;
} TCord;

typedef unsigned short wordbool;
typedef char ShortString[256];

typedef struct {
  short OCADMark;    // 3245 (hex 0cad)
  short SectionMark; // OCAD 6: 0
                     // OCAD 7: 7
                     // OCAD 8: 2 for normal files, 3 for course setting files
  short Version;     // 6 for OCAD 6, 7 for OCAD 7, 8 for OCAD 8
  short Subversion;  // number of subversion (0 for 6.00, 1 for 6.01 etc.)
  long FirstSymBlk;  // file position of the first symbol block
  long FirstIdxBlk;  // file position of the first index block
  long SetupPos;     // file position of the setup record 
  long SetupSize;    // size (in bytes) of the setup record
  long InfoPos;      // file position of the file
                     // information. The file information is
                     // stored as a zero-terminated string with
                     // up to 32767 characters + terminating
                     // zero
  long InfoSize;     // size (in bytes) of the file information
  long FirstStIndexBlk; // OCAD 8 only. file position of the first string index block
  long Reserved2;
  long Reserved3;
  long Reserved4;
} TFileHeader;


typedef struct {
  TCord lpMap;          // {Point on the map. All flag bits are set to 0}
  double Lat;           // {Latitude reported by the GPS receiver in this point (degrees)}
  double Long;          // {Longitude reported by the GPS receiver in this point (degrees)}
  char  Name[16];       // {name of the adjustment point as it appears in the Adjust GPS dialog box}
} TGpsAdjPoint;

typedef struct {
  double Zoom;          // {Zoom magnification as it appears in the View menu and in the status bar}
  TCord Offset;         // {Coordinate of the center of the screen. All flag bits are set to 0}
} TZoomRec;


typedef struct {
    TCord Offset; //          // Coordinate of the center of the screen when the file was
                              // last saved. All flag bits are set to 0}
    double rGridDist; //         {Grid distance for paper paper coordinates in mm}
    short WorkMode; //        {Mode when last the file was
                       //      last saved
                       //        5: freehand
                       //        6: straight
                       //        7: rectangular
                       //        8: circle
                       //        9: ellipse
                       //        10: curve
                       //        11: edit point
                       //        12: edit object}
    short LineMode; //        {drawing mode when the file was
                       //      last saved
                       //        5: freehand
                       //        6: straight
                       //        7: rectangular
                       //        8: circle
                       //        9: ellipse
                       //        10: curve}
    short EditMode; //        {edit mode when the file was
                       //      last saved
                       //        11: edit point
                       //        12: edit object}
    short ActSym; //          {selected symbol the file was last saved}
    double MapScale; //          {Map scale}
    double RealWorldOfsX; //     {Horizontal offset of real world coordinates in meters}
    double RealWorldOfsY; //     {Vertical offset of real world coordinates in meters}
    double RealWorldAngle; //    {Angle of real world coordinates in degrees}
    double RealWorldGrid; //     {Grid distance of real world coordinates in meters}
    double GpsAngle; //          {Angle of the GPS adjustment as displayed in the Adjust GPS dialog box}
    TGpsAdjPoint aGpsAdjust[12]; // {GPS adjustment points, this structure is defined below}
    long nGpsAdjust; //       {number of GPS adjustment points}
    double DraftScaleX; //       {Horizontal draft scale, not used in OCAD 8}
    double DraftScaleY; //       {Vertical draft scale, not uses in OCAD 8}
    TCord TempOffset; //         {Template offset. This defines the coordinates where the center of the template is displayed, not used in OCAD 8}
    char TemplateFileName[256]; // {file name of the template, not used in OCAD 8}
    wordbool TemplateEnabled; // {true if a template is opened, not used in OCAD 8}
    short TempResol; //       {resolution of the template file in DPI, not used in OCAD 8}
    double rTempAng; //          {Angle of the adjusted template, not used in OCAD 8}
    TCord Reserved1; //
    double Reserved2; //
    TCord PrLowerLeft; //        {lower left corner of the print window. All Flag bits are set to 0}
    TCord PrUpperRight; //       {upper right corner of the print window. All Flag bits are set to 0}
    wordbool PrGrid; //          {true if Print grid is activated in the Print dialog box}
    short PrGridColor; //     {Grid color selected in the Print dialog box}
    short PrOverlapX; //      {Horizontal overlap as defined in the Print Window dialog box unit is 0.01 mm}
    short PrOverlapY; //      {Vertical overlap as defined in the Print Window dialog box unit is 0.01 mm}
    double PrintScale; //        {Print scale}
    short PrIntensity; //     {Intensity as defined in the Printing Options dialog box}
    short PrLineWidth; //     {Line Width as defined in the Printing Options dialog box}
    wordbool PrReserved; //
    wordbool PrStdFonts; //      {OCAD 6: true if Standard fonts as PostScript fonts is activated
                         //       in the EPS Properties dialog box OCAD7/8: not used}
    wordbool PrReserved2; //
    wordbool PrReserved3; //
    TCord PartialLowerLeft; //   {lower left corner of Export Partial map window. All flag bits are set to 0}
    TCord PartialUpperRight; //  {upper right corner of the Export Partial map window. All flag bits are set to 0}
    double Zoom; //              {Zoom magnification as it appears in the View menu and in the status bar}
    TZoomRec ZoomHist[9]; // {Last 8 Zoom magnification for use in the Zoom out command. TZoomRec is explained below.}
    long nZoomHist; //        {number of magnificiations in ZoomHist}

    //      {OCAD 6: the setup ends here
    //      the following fields exist
    //      in OCAD 7 only}

    wordbool RealWorldCord; //   {true if real world coordinates are to be displayed}
    ShortString FileName; //     {used internally in temporary files.  The name of the original file is stored here}
    wordbool HatchAreas; //      {true if Hatch areas is active}
    wordbool DimTemp; //         {true if Dim template is active}
    wordbool HideTemp; //        {true if Hide template is active}
    short TempMode; //        {template mode
                       //        0: in the background
                       //        1: above a color}
    short TempColor; //       {the color if template mode is 1}
} TStp;
