#include <stdio.h>
#include <stdarg.h> //vfprintf, etc.
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <sys/stat.h>
#include <math.h>

#include "ascad.h"

int DEBUG = 0;
image_data_t img;
rect page;
source_t src;
char *tname;
char *desc;

void set_img(long cols, long rows, double res) {
  img.cols = cols;
  img.rows = rows;
  img.res_x = img.res_y = res;
}

init_src(double locx, double locy, int corner) {
  src.crop_length = -1;
  src.crop_box.x1 = src.crop_box.y1 = 99999;
  src.crop_box.x2 = src.crop_box.y2 = -99999;
  src.location_type = corner;
  src.location.x = locx;
  src.location.y = locy;
  src.scale = 1.0;
  src.rotate = 0.0;
  INIT_LIST_HEAD(&src.crop_path);
  INIT_LIST_HEAD(&src.col_trans_list);
}

void add_crop(double x, double y) {
  point *pt = malloc(sizeof(point));
  pt->x = x; pt->y = y;

  list_add_tail(&pt->list, &src.crop_path);

  src.crop_box.x1 = min(src.crop_box.x1, x);
  src.crop_box.y1 = min(src.crop_box.y1, y);
  src.crop_box.x2 = max(src.crop_box.x2, x);
  src.crop_box.y2 = max(src.crop_box.y2, y);

  if (src.crop_length < 0) src.crop_length = 0;
  src.crop_length++;
}
  
#define except_r(x1,y1,x2,y2, dx1,dy1,dx2,dy2) except(__FUNCTION__, x1,y1,x2,y2, dx1,dy1,dx2,dy2)

int except(char *name, int x1,int y1,int x2,int y2, double dx1, double dy1, double dx2, double dy2) {
  irect img_box;
  rect drawing_box;
  int err=0;

  printf("%s", name);

  if (calculate_range(&img, &src, &page, &img_box, &drawing_box)) {
    if (!err) printf(" [%s]", desc);
    printf("\n: out of range");
    err++;
  }

  if (img_box.x1 != x1 || img_box.y1 != y1 || img_box.x2 != x2 || img_box.y2 != y2) {
    if (!err) printf(" [%s]", desc);
    printf("\n: img_box expected (%ld,%ld)-(%ld,%ld), got (%ld,%ld)-(%ld,%ld)", x1, y1, x2, y2, img_box.x1, img_box.y1, img_box.x2, img_box.y2);
    err++;
  }

  if (drawing_box.x1 != dx1 || drawing_box.y1 != dy1 || drawing_box.x2 != dx2 || drawing_box.y2 != dy2) {
    if (!err) printf(" [%s]", desc);
    printf("\n: drawing_box expected (%.3f,%.3f)-(%.3f,%.3f), got (%.3f,%.3f)-(%.3f,%.3f)", dx1, dy1, dx2, dy2, drawing_box.x1, drawing_box.y1, drawing_box.x2, drawing_box.y2);
    err++;
  }

  if (err == 0) printf(" ok");
  printf("\n");
  return err;
}

/*====== 0: location */

int test0a() { desc = "location = lounas";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  return except_r(0,0,20,10, 0,0,20,10);
}

int test0b() { desc="location = kaakko";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(20, 0, LOCATION_LOWER_RIGHT);
  return except_r(0,0,20,10, -20,0,0,10);
}

int test0c() { desc="location = luode";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 10, LOCATION_UPPER_LEFT);
  return except_r(0,0,20,10, 0,-10,20,0);
}

int test0d() { desc="location = koillinen";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(20, 10, LOCATION_UPPER_RIGHT);
  return except_r(0,0,20,10, -20,-10,0,0);
}

int test0e() { desc="location = keskusta";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(10, 5, LOCATION_CENTER);
  return except_r(0,0,20,10, -10,-5,10,5);
}


/*====== 10: resolution */

int test10a() { desc="Resolution = 2.0";
  DEBUG = 0;
  set_img(20, 10, 2.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  return except_r(0,0,20,10, 0,0,10,5);
}

int test10b() { desc="Resolution = 0.1";
  DEBUG = 0;
  set_img(20, 10, 0.1);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  return except_r(0,0,20,10, 0,0,200,100);
}

/*====== 20: scale */

int test20a() { desc="scale = 200%";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  src.scale = 2.0;
  return except_r(0,0,20,10, 0,0,40,20);
}

int test20b() { desc="scale = 50%";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  src.scale = 0.5;
  return except_r(0,0,20,10, 0,0,10,5);
}

int test20c() { desc="scale = 200% + resolution = 2.0";
  DEBUG = 0;
  set_img(20, 10, 2.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  src.scale = 2.0;
  return except_r(0,0,20,10, 0,0,20,10);
}

/*====== 30: crop box */

int test30a() { desc="cropping rect (right)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  add_crop(0,0); add_crop(10,10);
  return except_r(0,0,10,10, 0,0,10,10);
}

int test30b() { desc="cropping rect (left)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  add_crop(10,0); add_crop(20,10);
  return except_r(10,0,20,10, 0,0,10,10);
}

int test30c() { desc="cropping rect (up)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  add_crop(0,0); add_crop(20,5);
  return except_r(0,5,20,10, 0,0,20,5);
}

int test30d() { desc="cropping rect (bottom)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  add_crop(0,5); add_crop(20,10);
  return except_r(0,0,20,5, 0,0,20,5);
}

int test30e() { desc="cropping rect (center)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  add_crop(2,1); add_crop(18,7);
  return except_r(2,3,18,9, 0,0,16,6);
}

int test30f() { desc="cropping path";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  add_crop(5,2);
  add_crop(15,5);
  add_crop(12,15);
  add_crop(7,10);
  return except_r(5,0,15,8, 0,0,10,8);
}

int test30g() { desc="cropping rect + resolution";
  DEBUG = 0;
  set_img(20, 10, 0.5);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  add_crop(0,0); add_crop(20,10);
  return except_r(0,5,10,10, 0,0,20,10);
}

int test30h() { desc="cropping rect + scale";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  add_crop(0,0); add_crop(10,10);
  src.scale = 2.0;
  return except_r(0,0,10,10, 0,0,20,20);
}

/*====== 40: page bounding box */

int test40a() { desc="page bounding box (right crop)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(190, 0, LOCATION_LOWER_LEFT);
  return except_r(0,0,10,10, 0,0,10,10);
}

int test40b() { desc="page bounding box (left crop)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(10, 0, LOCATION_LOWER_RIGHT);
  return except_r(10,0,20,10, -10,0,0,10);
}

int test40c() { desc="page bounding box (up crop)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 95, LOCATION_LOWER_LEFT);
  return except_r(0,5,20,10, 0,0,20,5); /* reverser rows! */
}

int test40d() { desc="page bounding box (bottom crop)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(20, 5, LOCATION_UPPER_RIGHT);
  return except_r(0,0,20,5, -20,-5,0,0); /* reversed rows! */
}

int test40e() { desc="page bounding box (bottom-left crop)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(10, 5, LOCATION_UPPER_RIGHT);
  return except_r(10,0,20,5, -10,-5,0,0);
}

int test40f() { desc="page bounding box (left and bottom crop) + scale2.0";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(20, 10, LOCATION_UPPER_RIGHT);
  src.scale = 2.0;
  return except_r(10,0,20,5, -20,-10,0,0);
}

int test40g() { desc="page bounding box + scale (right and up not crop)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(190, 95, LOCATION_LOWER_LEFT);
  src.scale = 0.5;
  return except_r(0,0,20,10, 0,0,10,5);
}

int test40h() { desc="page bounding box + cropping";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(10, 5, LOCATION_UPPER_RIGHT);
  add_crop(15,0); add_crop(20,8);
  return except_r(15,2,20,7, -5,-5,0,0);
}

/*====== 50: basic rotation */

int test50a() { desc="rotation 180";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(20, 10, LOCATION_LOWER_LEFT);
  src.rotate = 180.0;
  return except_r(0,0,20,10, 0,0,20,10);
}

int test50b() { desc="rotation -90";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(10, 20, LOCATION_LOWER_LEFT);
  src.rotate = -90.0;
  return except_r(0,0,20,10, 0,0,20,10);
}

/*====== 60: rotation + page bounding */

int test60a() { desc="rotation 180 + page crop (left)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(10, 10, LOCATION_LOWER_LEFT);
  src.rotate = 180.0;
  return except_r(0,0,10,10, 0,0,10,10);
}

int test60b() { desc="rotation -90 upper-left + page crop (up)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 90, LOCATION_UPPER_LEFT);
  src.rotate = -90.0;
  return except_r(0,0,10,10, 0,-10,10,0);
}

int test60c() { desc="rotation 10";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  src.rotate = 10.0;
  return except_r(0,0,20,10, 0,0,20,10);
}


int test60d() { desc="rotation 45 + page crop (bottom)";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  src.rotate = 45.0;
  return except_r(0,0,10,10, 0,0,10,10);
}

int test60e() { desc="rotation 45 + page crop (bottom) + scale";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  src.rotate = 45.0;
  src.scale = 2.0;
  return except_r(0,0,10,10, 0,0,20,20);
}


/*======*/

int testx() { desc="xxx";
  DEBUG = 0;
  set_img(20, 10, 1.0);
  init_src(0, 0, LOCATION_LOWER_LEFT);
  // src.scale = 1.0;
  // src.rotate = 0.0;
  // add_crop(0,0); add_crop(10,10);
  return except_r(0,0,20,10, 0,0,20,10);
}

int main(int argc, char* argv[])
{
  int failed = 0;

  page.x1 = page.y1 = 0.0;
  page.x2 = 200.0; page.y2 = 100.0;

  failed += test0a();
  failed += test0b();
  failed += test0c();
  failed += test0d();
  failed += test0e();

  failed += test10a();
  failed += test10b();

  failed += test20a();
  failed += test20b();
  failed += test20c();

  failed += test30a();
  failed += test30b();
  failed += test30c();
  failed += test30d();
  failed += test30e();
  failed += test30f();
  failed += test30g();
  failed += test30h();

  failed += test40a();
  failed += test40b();
  failed += test40c();
  failed += test40d();
  failed += test40e();
  failed += test40f();
  failed += test40g();
  failed += test40h();

  failed += test50a();
  failed += test50b();

  failed += test60a();
  failed += test60b();
  failed += test60c();
  failed += test60d();
  failed += test60e();

  return failed;
}

